
$ErrorActionPreference = 'Stop'

$config = @{}

$config.ServerServiceName = 'ABBYY.Server.FineReaderServer.14.0'
$config.StationServiceName = 'ABBYY.ProcStation.FineReaderServer.14.0'

$config.ProgramData = 'C:\ProgramData\ABBYY FineReader Server 14.0'
$config.ServerServiceSettingsXml = Join-Path $config.ProgramData 'OCRServerServiceSettings.xml'
$config.StationServiceSettingsXml = Join-Path $config.ProgramData 'OCRStationServiceSettings.xml'

$config.Bin32 = 'C:\Program Files (x86)\ABBYY FineReader Server 14.0\Bin'
$config.Bin64 = 'C:\Program Files\ABBYY FineReader Server 14.0\Bin'

function Check-Path($path)
{
    if (Test-Path -LiteralPath $path) { return $path }
    throw 'Path "{0}" does not exist' -f $path
}

function Remove-ItemIfExists($LiteralPath)
{
    if (Test-Path -LiteralPath $LiteralPath)
    {
        Remove-Item -LiteralPath $LiteralPath -ErrorAction Stop
    }
}

function Create-MissingDirectory($LiteralPath)
{
    if (Test-Path -LiteralPath $LiteralPath) { return }
    New-Item -Path $LiteralPath -ItemType Directory -Force -ErrorAction Stop | Out-Null
}

function Get-BinPath($fileName)
{
    $path = Join-Path $config.Bin32 $fileName
    if (Test-Path -LiteralPath $path) { return $path }
    $path = Join-Path $config.Bin64 $fileName
    if (Test-Path -LiteralPath $path) { return $path }
    throw '{0} not found' -f $fileName
}

function Get-ProgramDataPath($fileName)
{
    $path = Join-Path $config.ProgramData $fileName
    if (Test-Path -LiteralPath $path) { return $path }
    throw '{0} not found' -f $fileName
}

function Dump-EventLog
{
    [CmdLetBinding()]
    param
    (
        [Parameter(Mandatory = $true)]
        [string] $LogName,

        [Parameter(Mandatory = $false)]
        [int] $Newest,

        [Parameter(Mandatory = $false)]
        [System.DateTime] $After
    )

    $entries = Get-EventLog @PSBoundParameters

    '----------------------------------------------------------' | Write-Host
    'EventLog: {0}' -f $LogName | Write-Host
    '----------------------------------------------------------' | Write-Host
    foreach ($entry in $entries)
    {
        $lines = @()
        $lines += $entry.Message
        if ($entry.ReplacementStrings)
        {
            $lines += $entry.ReplacementStrings
        }

        $type = $entry.EntryType
        if ($type -eq 'Error') { $color = 'Red' }
        elseif ($type -eq 'Warning') { $color = 'Yellow' }
        else { $color = 'Cyan' }

        $type | Write-Host -NoNewline -ForegroundColor $color
        ': {0}' -f ($lines -join ' | ') | Write-Host        
    }
    '----------------------------------------------------------' | Write-Host
    '' | Write-Host
}

function Start-ProcessAndWait($FilePath, [string[]] $ArgumentList)
{
    'Starting process "{0}" {1}' -f $FilePath, ($ArgumentList -join ' ') | Write-Host
    
    $stdoutPath = [IO.Path]::GetTempFileName()
    $stderrPath = [IO.Path]::GetTempFileName()

    try {
        if (!$ArgumentList) 
        { 
            $p = Start-Process -FilePath $FilePath -PassThru -Wait -RedirectStandardOutput $stdoutPath -RedirectStandardError $stderrPath
        }
        else 
        { 
            $p = Start-Process -FilePath $FilePath -ArgumentList $ArgumentList -PassThru -Wait -RedirectStandardOutput $stdoutPath -RedirectStandardError $stderrPath
        }

        $message = 'Process exited with code {0}' -f $p.ExitCode
        $message | Write-Host

        $stdoutText = Get-Content -Raw -LiteralPath $stdoutPath -ErrorAction SilentlyContinue
        $stderrText = Get-Content -Raw -LiteralPath $stderrPath -ErrorAction SilentlyContinue

        if ($stdoutText)
            { "-- StdOut -----------------------------------", $stdoutText | Write-Host }
        if ($stderrText)
            { "-- StdErr -----------------------------------", $stderrText | Write-Host }
        if ($stdoutText -or $stderrText)
            { "---------------------------------------------" | Write-Host }

        if ($p.ExitCode -ne 0) { throw $message }
    } 
    finally 
    {
        Remove-Item -LiteralPath $stdoutPath -ErrorAction SilentlyContinue
        Remove-Item -LiteralPath $stderrPath -ErrorAction SilentlyContinue
    }
	
}

function Wrap-InQuotes($str) 
{
	return '"{0}"' -f $str 
}

function Install-Msi($FilePath, $AdditionalArguments)
{
	$logFilePath = [IO.Path]::GetTempFileName()
	try
	{
        $argumentList = @(
            '/quiet', '/passive', '/norestart',
            '/l*', (Wrap-InQuotes $logFilePath),
            '/i', (Wrap-InQuotes $FilePath)
        )
        if ($AdditionalArguments) { $argumentList += $AdditionalArguments }
		return Start-ProcessAndWait 'msiexec' -ArgumentList $argumentList
	}
	catch
	{
		Get-Content -LiteralPath $logFilePath -ErrorAction SilentlyContinue | Write-Host
		throw
    }
    finally
    {
        Remove-Item -LiteralPath $logFilePath -ErrorAction SilentlyContinue
    }
}

function Get-WindowsVersion
{
    $CurrenetVersionKey = 'HKLM:\SOFTWARE\Microsoft\Windows NT\CurrentVersion'
    $v = Get-ItemProperty $CurrenetVersionKey
    '{0} {1}.{2}.{3}.{4}' -f $v.ProductName, $v.CurrentMajorVersionNumber, $v.CurrentMinorVersionNumber, $v.CurrentBuild, $v.UBR
}

function Edit-Service
{
    [CmdLetBinding()]
    param
    (
        [Parameter(Mandatory = $true)]
        [string] $Name,
    
        [Parameter(Mandatory = $true, ParameterSetName = 'Start')]
        [switch] $Start,
    
        [Parameter(Mandatory = $true, ParameterSetName = 'Stop')]
        [switch] $Stop,
    
        [Parameter(Mandatory = $false)]
        [switch] $DisableAutoStart
    )
    
    $timeout = [TimeSpan]::FromSeconds(30)
    $service = Get-Service -Name $Name -ErrorAction Stop
    
    foreach ($s in $service)
    {
        if ($Start -and $s.Status -ne 'Running')
        {
             $s.Start() 
             $s.WaitForStatus('Running', $timeout)
        }
        elseif ($Stop -and $s.Status -ne 'Stopped') 
        { 
            $s.Stop() 
            $s.WaitForStatus('Stopped', $timeout)
        }
        if ($DisableAutoStart)
        {
            Set-Service -Name $s.Name -StartupType Manual -ErrorAction Stop
        }
        $s.Refresh()
        
        'Service "{0}" is {1}' -f $s.Name, $s.Status | Write-Host
    }    
}

function Start-ServiceMonitor($serviceName)
{
    $serviceMonitor = Check-Path (Join-Path $PSScriptRoot 'ServiceMonitor.exe')
    &$serviceMonitor $serviceName
}

function Configure-WCFService
{
    [CmdLetBinding()]
    param
    (
        [Parameter(Mandatory = $true)]
        [string] $LiteralPath,
    
        [Parameter(Mandatory = $true, ParameterSetName='Enable')]
        [string] $EndpointHostName,
    
        [Parameter(Mandatory = $true, ParameterSetName='Enable')]
        [string] $EndpointPortNumber,
        
        [Parameter(Mandatory = $false, ParameterSetName='Disable')]
        [switch] $Disable
    )
    
    $ConfigName = Split-Path -Leaf $LiteralPath
    
    if ($Disable)
    {
        Remove-ItemIfExists -LiteralPath $LiteralPath -ErrorAction Stop

        '{0}: WCF service disabled' -f $ConfigName | Write-Host
    }
    else 
    {
        $xml = [xml] @'
            <FileAnalyticsServiceSettings 
                BindingType="Tcp" 
                TransferMode="Streamed" 
                EndpointHostName="localhost" 
                EndpointPortNumber="3997" 
                IsMetadataExchangeEnabled="false" 
                MaxBufferSize="2147483647" 
                MaxReceivedMassageSize="2147483647" 
                OpenTimeoutSeconds="60" 
                CloseTimeoutSeconds="60" 
                SendTimeoutSeconds="300" 
                ReceiveTimeoutSeconds="300"
            />
'@
    
        $xml.FileAnalyticsServiceSettings.EndpointHostName = $EndpointHostName
        $xml.FileAnalyticsServiceSettings.EndpointPortNumber = $EndpointPortNumber
        $xml.Save($LiteralPath)
    
        '{0}: WCF service enabled on {1}:{2}' -f $ConfigName, $EndpointHostName, $EndpointPortNumber | Write-Host
    }
}

function Configure-DeveloperSettings
{
    [CmdLetBinding()]
    param
    (
        [Parameter(Mandatory = $true, ParameterSetName = 'EnableLogs')]
        [string] $LogDirectory,

        [Parameter(Mandatory = $false, ParameterSetName = 'EnableLogs')]
        [string] $LogTypes = 'RpcCall;Error;Test',
    
        [Parameter(Mandatory = $false, ParameterSetName = 'EnableLogs')]
        [switch] $EnableLogs,

        [Parameter(Mandatory = $false, ParameterSetName = 'DisableLogs')]
        [switch] $DisableLogs
    )

    Create-MissingDirectory $config.ProgramData

    $xmlPath = Join-Path $config.ProgramData 'DeveloperSettings.xml'
    if (!(Test-Path -LiteralPath $xmlPath))
    {
        $ConfigureOCRServer = Get-BinPath 'ConfigureOCRServer.exe'
        Start-ProcessAndWait $ConfigureOCRServer -ArgumentList '/CREATE_DEVELOPER_SETTINGS'
    }

    $xml = [xml] (Get-Content -Raw -LiteralPath $xmlPath -ErrorAction Stop)
    
    if ($EnableLogs)
    {
        Create-MissingDirectory $LogDirectory
        $appLogs = $xml.DeveloperSettings.LogsSettings.Application
        $appLogs.IsEnabled = 'true'
        $appLogs.Path = $LogDirectory
        $appLogs.StringTypes = $LogTypes

        'FRS logs "{0}" enabled: "{1}"' -f $LogTypes, $LogDirectory | Write-Host
    }
    elseif ($DisableLogs)
    {
        $appLogs = $xml.DeveloperSettings.LogsSettings.Application
        $appLogs.IsEnabled = 'false'

        'FRS logs disabled' | Write-Host
    }
    
    $xml.Save($xmlPath)
}

function Install-FineReaderServer
{
    [CmdLetBinding()]
    param
    (
        [Parameter(Mandatory = $false)]
        [string] $Source,

        [Parameter(Mandatory = $true)]
        [ValidateSet('x86', 'x64')]
        [string] $Platform,

        # Mutually exclusive options:

        [Parameter(Mandatory = $true, ParameterSetName = 'Redist')]
        [switch] $Redist,

        [Parameter(Mandatory = $true, ParameterSetName = 'LocalDB')]
        [switch] $LocalDB,

        [Parameter(Mandatory = $true, ParameterSetName = 'AOO')]
        [switch] $AOO,

        [Parameter(Mandatory = $true, ParameterSetName = 'FRS')]
        [string] $Features,

        [Parameter(Mandatory = $false)]
        [string[]] $AdditionalArguments
    )

    $startTime = [DateTime]::Now

    if (!$Source) { $Source = '.' }
    $Source = (Resolve-Path -LiteralPath $Source -ErrorAction Stop).Path

    function getArguments([string[]]$baseArguments)
    {
        $ret = @()
        if ($baseArguments) { $ret += $baseArguments }
        if ($AdditionalArguments) { $ret += $AdditionalArguments }
        return $ret
    }

    try {
        if ($Redist)
        {
            if ($Platform -ieq 'x86')
            {
                Start-ProcessAndWait (Join-Path $Source 'External Components\VC_Redist\vc_redist.x86.exe') -ArgumentList (getArguments '/quiet')
                Start-ProcessAndWait (Join-Path $Source 'External Components\VC_Redist_9\vcredist_x86.exe') -ArgumentList (getArguments '/q')
            }
            else
            {
                Start-ProcessAndWait (Join-Path $Source 'External Components\VC_Redist\vc_redist.x64.exe') -ArgumentList (getArguments '/quiet')
                Start-ProcessAndWait (Join-Path $Source 'External Components\VC_Redist_9\vcredist_x64.exe') -ArgumentList (getArguments '/q')
            }
        }
        elseif ($LocalDB)
        {
            if ($Platform -ieq 'x86')
            {
                Install-Msi (Join-Path $Source 'External Components\SqlLocalDB\SqlLocalDB_x86.msi') -AdditionalArguments (getArguments 'IACCEPTSQLLOCALDBLICENSETERMS=YES')
                Install-Msi (Join-Path $Source 'External Components\SqlLocalDB\msoledbsql_18.2.2.0_x86.msi') -AdditionalArguments (getArguments 'IACCEPTMSOLEDBSQLLICENSETERMS=YES')
            }
            else
            {
                Install-Msi (Join-Path $Source 'External Components\SqlLocalDB\SqlLocalDB_x64.msi') -AdditionalArguments (getArguments 'IACCEPTSQLLOCALDBLICENSETERMS=YES')
                Install-Msi (Join-Path $Source 'External Components\SqlLocalDB\msoledbsql_18.2.2.0_x64.msi') -AdditionalArguments (getArguments 'IACCEPTMSOLEDBSQLLICENSETERMS=YES')
            }
        }
        elseif ($AOO)
        {
            if ($Platform -ieq 'x86') 
            {
                $installDir = 'INSTALLDIR="C:\Program Files (x86)\ABBYY FineReader Server 14.0\Bin"'
                Install-Msi (Join-Path $Source 'ABBYY Digital Documents for FineReader Server 14.msi') -AdditionalArguments (getArguments $installDir)
            }
            else 
            {
                $installDir = 'INSTALLDIR="C:\Program Files\ABBYY FineReader Server 14.0\Bin"'
                Install-Msi (Join-Path $Source 'ABBYY Digital Documents for FineReader Server 14 (64 bit).msi') -AdditionalArguments (getArguments $installDir)
            }
        }
        elseif ($Features)
        {
            if ($Platform -ieq 'x86') 
                { $msi = 'ABBYY FineReader Server 14.0.msi' }
            else 
                { $msi = 'ABBYY FineReader Server 14.0 x64.msi' }

            Install-Msi (Join-Path $Source $msi) -AdditionalArguments (getArguments ('ADDLOCAL=' + $Features))
        }
    } catch {
        # Dump event logs on error
        Dump-EventLog 'Application' -After $startTime
        Dump-EventLog 'System' -After $startTime
        throw
    }
}

function Dump-InstalledPackages
{
    '----------------------------------------------------------' | Write-Host
    '| Installed packages' | Write-Host
    '----------------------------------------------------------' | Write-Host
    Get-Package | Format-Table -Property Name,Version -AutoSize -Wrap | Out-Host  
    '----------------------------------------------------------' | Write-Host 
}

function Dump-InstalledABBYYServices
{
    '----------------------------------------------------------' | Write-Host
    '| Installed ABBYY services' | Write-Host
    '----------------------------------------------------------' | Write-Host
    Get-Service -Name "ABBYY.*" | Format-Table -Property Name,Status -AutoSize -Wrap | Out-Host 
}

function Dump-WindowsVersion
{
    'OS: {0}' -f (Get-WindowsVersion) | Write-Host
}
