param
(
    [Parameter(Mandatory = $false)]
    [string] $ContainerHost,

    [Parameter(Mandatory = $false)]
    [switch] $Server,
	
	[Parameter(Mandatory = $false)]
    [string] $OnlineToken,
	
	[Parameter(Mandatory = $false)]
    [string] $SerialNumber,
	
    [Parameter(Mandatory = $false)]
    [switch] $Station,
    
    [Parameter(Mandatory = $false)]
    [switch] $Interactive,

    [Parameter(Mandatory = $false)]
    [switch] $Recovery,
	
	[Parameter(Mandatory = $false)]
    [switch] $WithLogs,

    [Parameter(Mandatory = $false)]
    [string] $serverLocation,

    [Parameter(Mandatory = $false)]
    [string] $ChiefAdminPassword
)

# How long to wait for a license to become available
$LicenseWaitTimeout = [TimeSpan]::FromMinutes(15)

if ($Recovery)
{
    # Just the interactive console
    powershell.exe -ExecutionPolicy ByPass -NoExit -Command ". .\common.ps1"
    return
}

if ($Interactive) { $ErrorActionPreference = 'SilentlyContinue' }
else { $ErrorActionPreference = 'Stop' }

. "$PSScriptRoot\common.ps1"

if ($OnlineToken -and $SerialNumber) { throw 'Parameters -SerialNumber and -OnlineToken cannot be used together' }
if (!$Server -and !$Station) { throw 'Specify -Server switch, -Station switch, or both' }

if ($OnlineToken -and !(Test-Path -LiteralPath $OnlineToken))
{
    throw 'The online token file "{0}" was not found in the container, please modify the Dockerfile according to the comments inside' -f $OnlineToken
}

$startTime = [DateTime]::Now

try {
    # Stop the services
    if ($Server) { Edit-Service -Name $config.ServerServiceName -Stop }
    if ($Station) { Edit-Service -Name $config.StationServiceName -Stop }

    # Configure logging
	if ($WithLogs) { Configure-DeveloperSettings -EnableLogs -LogDirectory 'C:\Logs' }
	else { Configure-DeveloperSettings -DisableLogs }

    # The server communicates its address to the stations for a return call, this should be the address of the host and not of the container
    if ($Server)
    {
        $configPath = Check-Path (Join-Path $config.ProgramData 'Configuration.xml')
        $cfg = [xml](Get-Content -Raw -LiteralPath $configPath -ErrorAction Stop)
        $qm = $cfg.OCRServer.QueueManager
        'Setting ExplicitServerHostName = "{0}"' -f $ContainerHost | Write-Host
        if (!$ContainerHost) { [void]$qm.RemoveAttribute('ExplicitServerHostName') }
        else { [void]$qm.SetAttribute('ExplicitServerHostName', $ContainerHost) }
        $cfg.Save($configPath)
    }

    # Activate the license
    if ($Server -and ($SerialNumber -or $OnlineToken))
    {
        $LicenseManagerConsole = Get-BinPath 'LicenseManager.Console.exe'

		if ($OnlineToken)
		{
			Start-ProcessAndWait $LicenseManagerConsole -ArgumentList ('/LoadOnlineToken:"{0}"' -f $OnlineToken)
		}
		else
		{ 
			Start-ProcessAndWait $LicenseManagerConsole -ArgumentList ('/Activate:"{0}"' -f $SerialNumber) 
		}

        $swLicense = [System.Diagnostics.StopWatch]::StartNew()
        while ($true)
        {
            $licenses = &$LicenseManagerConsole /ListAvailableLicenses | Where-Object { $_.Trim() -imatch '^([A-Z0-9]{4}\-){5}[A-Z0-9]{4}$' }
            if ($licenses)
            { 
                'A license is available: {0}' -f ($licenses -join ', ') | Write-Host
                'Elapsed: {0}' -f $swLicense.Elapsed | Write-Host
                break
            }
            
            if ($swLicense.Elapsed -gt $LicenseWaitTimeout) { throw 'Wait for license timed out' }
            'Waiting for an available license ...' | Write-Host
            Start-Sleep -Seconds 15
        }
    }
    if ($Interactive)
    {
        if ($Station) { Edit-Service -Name $config.StationServiceName -Start }
        if ($Server) { Edit-Service -Name $config.ServerServiceName -Start }
        powershell.exe -ExecutionPolicy ByPass -NoExit -Command ". .\common.ps1"
        return
    }

    else
    {
        if ($Server -and $Station)
        {
            Edit-Service -Name $config.StationServiceName -Start
            Start-ServiceMonitor $config.ServerServiceName
            Edit-Service -Name $config.StationServiceName -Stop
        }
        elseif ($Server)
        {
            Start-ServiceMonitor $config.ServerServiceName
        }
        else
        {
            Edit-Service -Name $config.StationServiceName -Start
            if($serverLocation){
                $FRSCli = Get-BinPath 'FRS.Cli.exe'
                Start-ProcessAndWait $FRSCli -ArgumentList ('create-station /server:{0} /name:{1} /location:{2} /IgnoreVersion:true /password:{3}' -f $serverLocation, $env:COMPUTERNAME, $containerHost, $ChiefAdminPassword)
            }
            Start-ServiceMonitor $config.StationServiceName
        }
    }
} catch {
    # Dump event logs on error
    Dump-EventLog 'Application' -After $startTime
    Dump-EventLog 'System' -After $startTime
    Dump-EventLog 'ABBYY FineReader Server' -After $startTime
    throw
}